import * as zlib from 'zlib';
import { ISsnFileEntry } from '../interfaces/ISsnFileEntry';
import decryptFile from './decryption/decryptFile';
import ByteReader from './extractFileByteReader';

/** Extracts the given file from the given DataView array and returns it as an ArrayBuffer.
 * Will throw an error when end of final DataView is reached.
 */
export default async function extractFile(file: ISsnFileEntry, dvArray: DataView[]): Promise<ArrayBuffer> {
  //Use ByteReader for reading a uint8 and seeking forward across DataView boundaries
  const byteReader = new ByteReader(dvArray, file.diskNumberStart, file.offset);

  //Local file header signature must be 0x04034B50
  if (byteReader.readByte() !== 0x50 || byteReader.readByte() !== 0x4B || byteReader.readByte() !== 0x03 || byteReader.readByte() !== 0x04) {
    throw new Error('Local file header had wrong magic');
  }
  //All fields in the local file header are copies of the central file header, so we can skip them.
  //FIXME: Maybe we should actually read these fields to verify that they are identical?
  byteReader.seek(22);
  const localFilenameSize = byteReader.readByte() + (byteReader.readByte() << 8);
  const localExtraSize = byteReader.readByte() + (byteReader.readByte() << 8);
  byteReader.seek(localFilenameSize + localExtraSize);

  //Extract actual file contents
  let dvFinal = byteReader.extractDv(file.comprSize);

  //Decrypt file if necessary
  if (file.decryptionKeys !== undefined) {
    dvFinal = decryptFile(dvFinal, file.comprSize, file.decryptionKeys);
  }

  //Uncompress file
  const uncompressedBuffer: Buffer = await new Promise((resolve, reject) => {
    zlib.inflateRaw(dvFinal, (error, result) => {
      if (error !== null) {
        return reject(error);
      }
      resolve(result);
    });
  }) as Buffer;
  return uncompressedBuffer.buffer as ArrayBuffer;
}
